//  name		hanoitower.cpp
//	autor		Hong Phuc Bui		KIB-3464938
//	datum		19.02.2008
//	copyleft	GNU
//	beschreibung	Tower of Hanoi, Demo (benutzt moveproc)
//	version		1.0
//	build		g++ -Wall -pedantic -o hnt -lncurses hanoitower.cpp
//	depend		libncurses5-dev



#include <iostream>
#include <cmath>		// pow()
using namespace std;

#include <ncurses.h>
#include <unistd.h>

// Konstanten:
#define FLOOR 16
#define L_RAND 1
#define MAX_DISK 12
#define MAXMOVE 4095

class disk
{
	public :
		int r;		//radius
		int h;		//hight
		int p;		//p = 0,1,2

		void hold_me ();
		void move_me(int position);
		void drop_me(int hight);
		void init (int radius, int hight, int position);
		void display();
		//private :

		void clear();
};

class tower
{
	public :
		int h;	//max disk
		int tp; //actuaell Anzahle der Teller

		void init (int n);
		void showme (int i);
		bool is_empty();
		bool is_full ();
		bool get_disk(disk ndisk);
		bool get_dsok( disk ndisk );
		bool give_disk(disk &dsk);
		//private :
		disk a[MAX_DISK];
};


struct bring
{
	int von;
	int nach;
};

void tower::init (int n)
{
	h = n;
	tp = 0;
}

bool tower::is_empty()
{	
	if (tp == 0)
	{
		return true;
	}else
	{
		return false;
	}
}

bool tower::get_dsok( disk ndisk )
{
	if ( is_empty() || ndisk.r > a[tp-1].r)
		//wenn Radius der Scheibe groesser als die aktuelle Scheibe
		return true;
	else 
		return false;
}


bool tower::is_full()
{
	if (tp == h)
	{
		return true;
	}else
	{
		return false;
	}
}

bool tower::get_disk(disk ndisk)
{
	if ((is_empty() )|| (ndisk.r < a[tp-1].r) )
	{
		a[tp] = ndisk;
		a[tp].r = ndisk.r;
		a[tp].p = ndisk.p;
		a[tp].h = ndisk.h;
		tp++;
		return true;
	}else 
	{
		return false;
	}
}

bool tower::give_disk(disk &dsk)
{
	if ( is_empty() )
	{
		return false;
	}
	else
	{
		tp--;
		dsk=a[tp];
		dsk.r=a[tp].r;
		dsk.p=a[tp].p;
		dsk.h=a[tp].h;

		return true;
	}
}

void tower::showme(int i)
{
	int y= FLOOR- (h+2);
	int x= (L_RAND + i*26+13 );
	mvvline(y,x,'H',h+1);
}

//------------------------------------------------------
void disk::init(int radius, int hight, int position)
{
	r = radius;
	h = hight;
	p = position;
}

void disk::display()
{
	int x = (L_RAND + p*26 +1);
	int y = FLOOR -(h+1);
	move (y,x);

	start_color();
	init_pair(1, COLOR_CYAN, COLOR_BLACK);
	attron(COLOR_PAIR(1));
	for (int i = - MAX_DISK; i <= MAX_DISK; i++)
	{
		if (i <-r ) {printw(" ");continue;}
		if (i==-r ) {printw("[");continue;}
		if (i < 0 ) {printw("#");continue;}
		if (i ==0 ) {printw("H");continue;}
		if (i < r ) {printw("#");continue;}
		if (i ==r ) {printw("]");continue;}
		printw(" ");
	}
	attroff(COLOR_PAIR(1));
}

void disk::clear()
{
	int x = L_RAND + p*26 +1;
	int y = FLOOR -(h+1);
	mvhline(y,x,' ',25);
}

void disk::hold_me()
{
	clear();
	h = 14;
	display();
}
void disk::move_me(int position)
{
	clear();
	p=position;
	display();
}
void disk::drop_me(int hight)
{
	clear();
	h = hight;
	display();
}

///////////////////////////////////////////////////////
void move_disk(tower &from, tower &to, bring mstore)
{
	disk tmp;	
	from.give_disk(tmp);

	int y= FLOOR - (tmp.h+1);
	int x= (L_RAND + mstore.von*26+13 );

	//getch();
	refresh();
	sleep(1);
	tmp.hold_me();
	mvprintw(y,x,"H");

	//getch();
	refresh();
	sleep(1);
	tmp.move_me(mstore.nach);

	int hight = to.tp;
	hight++;
	//getch();
	refresh();
	sleep(1);
	tmp.drop_me(hight);
	to.get_disk(tmp);

}

void moveproc(bring mstore[], int n, int q, int t)
{
	static int i = 0;
	if (n ==1)
	{
		mstore[i].von = q-1;
		mstore[i].nach = t-1;
		i++;
		return;
	}
	else
	{ 
		int tmp = 0;
		if (q+t==3)
		{ 
			tmp = 3;
		}else 
		{ 
			if(q+t==4)
			{ 
				tmp = 2;
			}else tmp = 1;
		}

		moveproc(mstore, n-1, q, tmp);
		mstore[i].von = q-1;
		mstore[i].nach = t-1;
		i++;
		moveproc(mstore, n-1, tmp, t);

	}
}


int main(int argc,char **argv)
{
	int n = 0;

	cout << "Willkommen in Hanoi." << endl;

	if (argc<=1)
	{
		cout << "Wie viele Scheiben? ";
		cin >> n;
	}
	else
		n=atoi(argv[1]);

	if (n>=MAX_DISK)
	{
		cout << "zuviele Scheiben" << endl;
		return 1;
	}

	if (n<1)
	{
		cout << "zuwenig Scheiben" << endl;
		return 1;
	}


	int m = int ( pow(2.0,int(n) ) -1 );//anzahle der Schritte
	bring m_store[MAXMOVE];
	moveproc(m_store, n, 1, 2);	

	disk d[MAX_DISK];
	tower t[3];
	for (int i = 0; i < 3; i++)
	{
		t[i].init(n);
	}

	for (int i = 0; i < n; i++ )
	{
		d[i].init(n-i,i+1,0);

		t[0].get_disk( d[i] );

	}

	initscr();

	mvhline(FLOOR-1, L_RAND, '=', 78);	//vertiakal line


	for (int i = 0; i < 3; i++)			//3 towers
	{
		t[i].showme(i);
	}

	for (int i = 0; i < n; i++)		//Initalisierung (Anfangssituation)
	{
		d[i].display();
	}

	for (int i = 0; i< m; i++)
	{
		int quell  = m_store[i].von;
		int target = m_store[i].nach;
		move_disk(t[quell], t[target], m_store[i]);
	}

	endwin();
	return 0;
}
