//  name		hanoitower.cpp
//	autor		Hong Phuc Bui		KIB-3464938
//	datum		06.01.2008
//	copyleft	GNU
//	beschreibung	Tower of Hanoi, Hinweis
//	version		0.0
//	build		g++ -o hnt -lncurses hanoitower.cpp
//	depend		libncurses5-dev



#include <iostream>
using namespace std;

#include <ncurses.h>
#define FLOOR 16
#define L_RAND 1
#define MAX_DISK 12


class tower
{
	public :
		int h;
		int tp;
		
		void init (int n);
		void showme (int i);
		bool is_empty();
		bool is_full ();
		bool get_disk(int disk_index);
		bool get_dsok(int dr);
		bool give_disk(int &disk_index);
	//private :
		int a [MAX_DISK];
};

void tower::init (int n)
{
	h = n;
	tp = 0;
}

bool tower::is_empty()
{	
	if (tp == 0)
	{
		return true;
	}else
	{
		return false;
	}
}

bool tower::get_dsok( int disk_index )
{
	if ( is_empty() || disk_index > a[tp-1])
		return true;
	else 
		return false;
}


bool tower::is_full()
{
	if (tp == h)
	{
		return true;
	}else
	{
		return false;
	}
}

bool tower::get_disk(int disk_index)
{
	if ((is_empty() )|| (disk_index > a[tp-1]) )
	{
		a[tp] = disk_index;
		tp++;
		return true;
	}else 
	{
		return false;
	}
}

bool tower::give_disk(int &disk_index)
{
	if ( is_empty() )
	{
		return false;
	}else
	{
		tp--;
		disk_index=a[tp];
		return true;
	}
}

void tower::showme(int i)
{
	int y= FLOOR- (h+2);
	int x= (L_RAND + i*26+13 );
	mvvline(y,x,'H',h+1);
}

//------------------------------------------------------
class disk
{
	public :
		int r;		//radius
		int h;		//hight
		int p;		//p = 0,1,2
		
		void hold_me (int n);
		void move_me(int position);
		void drop_me(int hight);
		void init (int radius, int hight, int position);
		void display();
	//private :
		
		void clear();
};

void disk::init(int radius, int hight, int position)
{
	r = radius;
	h = hight;
	p = position;
}

void disk::display()
{
	
	int x = (L_RAND + p*26 +1);
	int y = FLOOR -(h+1);
	move (y,x);
	
	start_color();
	init_pair(1, COLOR_CYAN, COLOR_BLACK);
	attron(COLOR_PAIR(1));
	for (int i = - MAX_DISK; i <= MAX_DISK; i++)
	{
		if (i <-r ) {printw(" ");continue;}
		if (i==-r ) {printw("[");continue;}
		if (i < 0 ) {printw("#");continue;}
		if (i ==0 ) {printw("H");continue;}
		if (i < r ) {printw("#");continue;}
		if (i ==r ) {printw("]");continue;}
		printw(" ");
	}
	attroff(COLOR_PAIR(1));
}

void disk::clear()
{
	int x = L_RAND + p*26 +1;
	int y = FLOOR -(h+1);
	mvhline(y,x,' ',25);
}

void disk::hold_me(int n)
{
	clear();
	h = 14;
	display();
}
void disk::move_me(int position)
{
	clear();
	p=position;
	display();
}
void disk::drop_me(int hight)
{
	clear();
	h = hight;
	display();
}
//global fuction --------------------------------------------
bool move_disk(disk &d)
//gibt true wenn kein zwischen break zurueck;
{
	int position = d.p;
	int k = 0;
	noecho();
	bool doit = true;
	while (doit)
	{
		k = getch();
		switch (k){
			case 'd':{
				position = (position+1)%3;
				d.move_me(position);
			}break;
			case 'a':{
				position = (position+2)%3;
				d.move_me(position);
			}break;
			case 's':{
				doit = false;
				
			}break;
			case 'w':{
				doit = false;
				
			}break;
			case 'b':{
				char confirm = 0;
				printw ("Do you want to quit? Press 'j' to exit");
				confirm = getch();
				if (confirm =='j')
				{
					return false;		//nicht vollstandig
				}
			}break;
		}
	}
	return true;
}

///////////////////////////// MAIN FUNKTION ///////////////////////////////////
int main()
{
	disk cursor;
	cursor.init(1,-1,0);
	
	cout <<"How many disks would you like to play? ";
	int n;
	cin >>n;
	
	disk d[n];
	
	tower t[3];
	for (int i = 0; i < 3; i++)
	{
		t[i].init(n);
	}
	
	
	
	for (int i = 0; i < n; i++ )
	{
		d[i].init(n-i,i+1,0);
		
		t[0].get_disk( i );
		
	}
	

	int position = 0;
	int n_position=0;
	int dr_hight = 0;
	int swap = 0;
	bool win = false;
	bool is_choose = false;
	bool is_continue = true;
	
	initscr();
	
	mvhline(FLOOR-1, L_RAND, '=', 78);			//vertiakal line
	
	
	for (int i = 0; i < 3; i++)					//3 towers
	{
		t[i].showme(i);
	}
	
	
	for (int i = 0; i < n; i++)
	{
		d[i].display();
	}
	
	cursor.display();
	
	
	while (!win)
	{
		do
		{
			is_continue = move_disk(cursor);
			if ( !(is_continue))
			{
				goto end_loop;	
			}else
			{
				position = cursor.p;
				is_choose = t[position].is_empty();
			}
		}
		while (is_choose);
		
			t[position].give_disk(swap);
			int h = d[swap].h;
			
			d[swap].hold_me(n);
			int y= FLOOR- (h+1);
			int x= (L_RAND + position*26+13 );
			mvprintw(y,x,"H");
		do
		{
			is_continue = move_disk(d[swap]);
			if ( !(is_continue))
			{
				goto end_loop;	
			}else
			{
				n_position = d[swap].p;
				is_choose =  !( t[n_position].get_dsok(n - (d[swap].r)) );
			}
		}while (is_choose);
			
			t[n_position].get_disk( n - (d[swap].r)  );
		
			dr_hight = t[n_position].tp;
			d[swap].drop_me(dr_hight);
		
		win = t[1].is_full() || t[2].is_full();
		
	}
	end_loop:
	if (win)
	{
		printw("You win. Press any key to quit");
	}
	else 
	{
		printw("You lost. Press any key to quit");
	}
	getch();
	endwin();
	return 0;
}

